#include <stdio.h>
#include <fstream>
using std::ifstream;
using std::ios;
typedef unsigned char byte;
typedef unsigned short word;

struct msheader
{
    char id[10];
    int ver;
}__attribute__((packed));
struct msvertex
{
    byte flags;
    float vertex[3];
    char boneid;
    byte refcount;
}__attribute__((packed));      
struct mstriangle
{
    word flags,vertex[3];
    float vertexnormals[3][3],s[3],t[3];
    byte smoothinggroup,groupindex;
}__attribute__((packed));
struct msgroup
{
    byte flags;
    char *name;
    word ntriangles;
    word *triangle;
    char material;
}__attribute__((packed));        
struct msmaterial
{
    char name[32];
    float ambient[4];
    float diffuse[4];
    float specular[4];
    float emission[4];
    float shininess;	
    float transparency;
    byte mode;
    char texture[128];
    char alphamap[128];
}__attribute__((packed));    
struct mscoll
{
    char *buffer;
    long filesize;
    msheader *header;
    word nvertex,ntriangles,ngroups,nmaterials;
    msvertex *vertex;
    mstriangle *triangle;
    msgroup *group;
    msmaterial *material;
};
void LoadModel(mscoll *model,const char *filename)
{
    #define READWORD(P) P = *(word *)ind; ind+=sizeof(word)
    long i; msgroup *cg;
    ifstream inp(filename, ios::in | ios::binary);
    if(!inp.fail()){        
        inp.seekg( 0, ios::end );
        model->filesize = inp.tellg();
        inp.seekg( 0, ios::beg );
	    model->buffer = new char[model->filesize];
	    inp.read(model->buffer,model->filesize);
	    inp.close();
     	char *ind=model->buffer;
	    model->header = (msheader*)ind; ind+=sizeof(msheader);
	    READWORD(model->nvertex);
	    model->vertex = (msvertex*)ind; ind+=model->nvertex * sizeof(msvertex);
	    READWORD(model->ntriangles);
        model->triangle = (mstriangle*)ind; ind+=model->ntriangles * sizeof(mstriangle);
	    READWORD(model->ngroups);
	    model->group = new msgroup[model->ngroups]; cg = model->group;
	    for (i=0;i<model->ngroups;i++,cg++){
	        cg->flags = *(byte *)ind; ind+=sizeof(byte);
	        cg->name = ind; ind+=32;
	        READWORD(cg->ntriangles);
	        cg->triangle = (word *)ind; ind+=sizeof(word)*cg->ntriangles;
	        cg->material = *ind; ind++;
	    }    
	    READWORD(model->nmaterials);
	    model->material = (msmaterial*)ind; ind+=model->nmaterials * sizeof(msmaterial);
    }    
} 
void DelModel(mscoll *model)
{
    delete[] model->group;
    delete[] model->buffer;
}    
void DrawModel(mscoll *model)
{
    msgroup *cg = model->group;
  //  GLboolean texEnabled = glIsEnabled( GL_TEXTURE_2D );
    for(int i=0;i<model->ngroups;i++,cg++){
        int index = cg->material;
        msmaterial *cm = model->material; cm+=index;
        word *cti = cg->triangle;
        if (index>=0) {
     /*       glMaterialfv(GL_FRONT,GL_AMBIENT,cm->ambient);
            glMaterialfv(GL_FRONT,GL_DIFFUSE,cm->diffuse);
            glMaterialfv(GL_FRONT,GL_SPECULAR,cm->specular);
            glMaterialfv(GL_FRONT,GL_EMISSION,cm->emission);
            glMaterialf(GL_FRONT,GL_SHININESS,cm->shininess);
/*            if (cm->texture!=NULL){	
                
                glBindTexture( GL_TEXTURE_2D, cm->texture );
				glEnable( GL_TEXTURE_2D );
            } else */ //glDisable(GL_TEXTURE_2D);
        } else glDisable(GL_TEXTURE_2D); // no material!
        glBegin(GL_TRIANGLES);
        glEnable( GL_TEXTURE_2D );
          for(int j=0;j<cg->ntriangles;j++,cti++){
            mstriangle *ct = model->triangle; ct+=*cti;
            for (int k=0;k<3;k++){
                glNormal3fv(&ct->vertexnormals[k][0]);
                glTexCoord2f(ct->s[k],ct->t[k]);
                glVertex3fv(model->vertex[ct->vertex[k]].vertex);
            }          
          }    
        glEnd();        
    }
//    if(texEnabled) glEnable( GL_TEXTURE_2D ); else glDisable( GL_TEXTURE_2D );    
}    

